%{
  open Ast
  open Scanner
%}

%token T_true
%token T_false
%token T_and
%token T_or
%token T_not
%token T_print
%token T_begin
%token T_end
%token T_for
%token T_to
%token T_do
%token T_if
%token T_then
%token T_else
%token T_let
%token T_while
%token T_byref
%token T_return
%token T_var

%token T_assign    /* "="  */
%token T_plus      /* "+"  */
%token T_minus     /* "-"  */
%token T_mult      /* "*"  */
%token T_div       /* "/"  */
%token T_lt        /* "<"  */
%token T_gt        /* ">"  */
%token T_neq       /* "<>" */
%token T_eq        /* "==" */

%token T_lparen    /* "("  */
%token T_rparen    /* ")"  */
%token T_lbrack    /* "{"  */
%token T_rbrack    /* "}"  */
%token T_comma     /* ","  */

%token <string> T_id
%token <int> T_num

%token T_eof

%nonassoc T_let
%nonassoc T_if
%nonassoc T_then
%nonassoc T_else
%left T_and T_or
%nonassoc T_assign T_lt T_gt T_eq T_neq
%left T_plus T_minus
%left T_mult T_div
%nonassoc UNARY T_not

%start program

%type <unit> program
%type <ast_fun_def list> fun_defs
%type <ast_fun_def> fun_def
%type <ast_param list> maybe_formal_params
%type <ast_param list> formal_params
%type <ast_param> param
%type <ast_stmt list> stmt_list
%type <ast_stmt> stmt
%type <ast_expr> expr
%type <ast_expr list> maybe_actual_params
%type <ast_expr list> actual_params

%%

program : fun_defs T_eof { ast_tree := Some $1 }

fun_defs:
  /* nothing */     { [] }
| fun_def fun_defs  { $1 :: $2 }

fun_def:
  T_id T_lparen maybe_formal_params T_rparen T_lbrack vars stmt_list T_rbrack { Fundef ($1, $3, $6, $7) }

vars:
  /* nothing */    { [] }
| T_var many_vars  { $2 }

many_vars :
  T_id                    { [$1] }
| T_id T_comma many_vars  { $1 :: $3 }

maybe_formal_params:
  /* nothing */  { [] }
| formal_params  { $1 }

formal_params:
  param                        { [$1] }
| param T_comma formal_params  { $1 :: $3 }

param:
  T_id            { Param $1 }
| T_byref T_id    { ParamByRef $2 }

stmt_list:
  /* nothing */   { [] }
| stmt stmt_list  { $1 :: $2 }

stmt:
  T_print expr  { Sprint $2 }
| T_let T_id T_assign expr  { Slet ($2, $4) }
| T_begin stmt_list T_end  { Sbegin $2 }
| T_for T_id T_assign expr T_to expr T_do stmt  { Sfor ($2, $4, $6, $8) }
| T_while expr T_do stmt  { Swhile ($2, $4) }
| T_if expr T_then stmt  { Sif ($2, $4, None) }
| T_if expr T_then stmt T_else stmt  { Sif ($2, $4, Some $6) }
| T_return expr  { Sreturn $2 }

expr:
  T_num  { Enum $1 }
| T_id  { Eid $1 }
| T_false { Ebool false }
| T_true { Ebool true }
| expr T_plus expr  { Eplus ($1, $3) }
| expr T_mult expr  { Emult ($1, $3) }
| expr T_minus expr  { Eminus ($1, $3) }
| expr T_div expr  { Ediv ($1, $3) }
| expr T_and expr  { Eand ($1, $3) }
| expr T_or expr  { Eor ($1, $3) }
| expr T_lt expr  { Elt ($1, $3) }
| expr T_gt expr  { Egt ($1, $3) }
| expr T_eq expr  { Eeq ($1, $3) }
| expr T_neq expr  { Eneq ($1, $3) }
| T_not expr  { Enot $2 }
| T_minus expr %prec UNARY { Eunary $2 }
| T_lparen expr T_rparen { $2 }
| T_id T_lparen maybe_actual_params T_rparen  { Eapp ($1, $3) }

maybe_actual_params:
  /* nothing */  { [] }
| actual_params  { $1 }

actual_params:
  expr  { [$1] }
| expr T_comma actual_params  { $1 :: $3 }

