{
  open Parser

  let num_lines = ref 1
}

let digit = ['0'-'9']
let letter = ['a'-'z' 'A'-'Z']
let white = [' ' '\t' '\r']

rule lexer = parse
  | "true"   { T_true   }
  | "false"  { T_false  }
  | "var"    { T_var    }
  | "and"    { T_and    }
  | "or"     { T_or     }
  | "not"    { T_not    }
  | "print"  { T_print  }
  | "begin"  { T_begin  }
  | "end"    { T_end    }
  | "for"    { T_for    }
  | "to"     { T_to     }
  | "do"     { T_do     }
  | "if"     { T_if     }
  | "then"   { T_then   }
  | "else"   { T_else   }
  | "let"    { T_let    }
  | "while"  { T_while  }
  | "byref"  { T_byref  }
  | "return" { T_return }
  | "="      { T_assign }
  | "+"      { T_plus   }
  | "-"      { T_minus  }
  | "*"      { T_mult   }
  | "/"      { T_div    }
  | "<"      { T_lt     }
  | ">"      { T_gt     }
  | "<>"     { T_neq    }
  | "=="     { T_eq     }
  | "("      { T_lparen }
  | ")"      { T_rparen }
  | "{"      { T_lbrack }
  | "}"      { T_rbrack }
  | ","      { T_comma  }
  | letter (letter | digit)* as ident  { T_id ident }
  | digit+ as intconst  { let num = int_of_string intconst in T_num num }
  | '\n'  { incr num_lines; lexer lexbuf }
  | white+ as blanks  { lexer lexbuf }
  | eof  { T_eof }
  | _ as chr  { Printf.eprintf "Lexical Error: '%c' at Ln %d.\n" chr !num_lines; lexer lexbuf }
